defmodule IndieWeb.Cache.Adapters.Cachex do
  @behaviour IndieWeb.Cache.Adapter
  @moduledoc """
  Wraps logic for using `Cachex` as the caching mechanism in `IndieWeb`.

  NOTE: This uses the cache name `indieweb`.
  """

  @expiry_seconds 10 * 60

  @impl true
  def get(key, default_value) do
    result = Cachex.get(:indieweb, key)

    case result do
      {:ok, value} when not is_nil(value) -> value
      _ -> default_value
    end
  end

  @impl true
  def delete(key) do
    {:ok, _} = Cachex.expire(:indieweb, key, -1)

    case Cachex.del(:indieweb, key) do
      {:ok, _} -> :ok
      err -> {:error, err}
    end
  end

  @impl true
  def set(key, value, options \\ []) do
    case Cachex.put(:indieweb, key, value) do
      {:ok, _} ->
        {:ok, true} =
          Cachex.expire(
            :indieweb,
            key,
            :timer.seconds(Keyword.get(options, :expire, @expiry_seconds))
          )

        :ok

      err ->
        {:error, err}
    end
  end
end
