defmodule IndieWeb.MixProject do
  use Mix.Project
  @description "Helpers and facilities for working with the IndieWeb."

  def project do
    [
      aliases: aliases(),
      app: :indieweb,
      name: "IndieWeb",
      description: "A collection of higher-order functions for interacting with the IndieWeb.",
      version: "0.0.64",
      elixir: "~> 1.7",
      elixirc_paths: elixirc_paths(Mix.env()),
      start_permanent: Mix.env() == :prod,
      build_embedded: Mix.env() == :prod,
      description: @description,
      package: package(),
      test_coverage: [tool: ExCoveralls],
      preferred_cli_env: [
        ci: :test,
        test: :test,
        "coveralls.detail": :test
      ],
      deps: deps(),
      homepage_url: "https://indieweb.org/",
      docs: [
        source_url: "https://git.jacky.wtf/indieweb/elixir",
        source_url_pattern:
          "https://git.jacky.wtf/indieweb/elixir/src/branch/master/%{path}#L%{line}",
        logo: "priv/static/images/logo.png",
        extras: Path.wildcard("docs/*.markdown")
      ]
    ]
  end

  def application do
    [
      mod: {IndieWeb.Application, []},
      extra_applications: [:logger, :cachex, :runtime_tools, :floki, :mnesia]
    ]
  end

  defp deps do
    [
      {:apex, "~> 1.2.1", only: [:dev, :test]},
      {:castore, "~> 0.1.0"},
      {:cachex, "~> 3.1.0"},
      {:calendar, "~> 1.0.0"},
      {:credo, "~> 0.8", only: [:dev, :test]},
      {:ex_doc, "~> 0.19", only: [:dev, :test]},
      {:dialyxir, "~> 1.0", only: [:dev], runtime: false},
      {:excoveralls, "~> 0.10.0", only: [:test]},
      {:exvcr, "~> 0.10.0", only: :test},
      {:faker, "~> 0.12.0", only: :test},
      {:fake_server, "~> 2.1", only: :test},
      {:jason, "~> 1.0"},
      {:tesla, "1.3.0"},
      {:microformats2, "~> 0.6.0"},
      {:hackney, "~> 1.17.0"},
      {:explode, "~> 1.1.0"},
      {:plug, "~> 1.8", optional: true}
    ]
  end

  defp package do
    [
      name: "indieweb",
      description:
        "Collection of common IndieWeb utilites like authorship resolution, Webmention, post type discovery and IndieAuth.",
      licenses: ["APGL v3.0"],
      links: %{
        "Source Code" => "https://git.jacky.wtf/indieweb/elixir",
        "IndieWeb" => "https://indieweb.org",
        "IndieAuth spec" => "https://indieauth.spec.indieweb.org"
      }
    ]
  end

  defp elixirc_paths(:test), do: ["lib", "test/support"]
  defp elixirc_paths(_), do: ["lib"]

  defp aliases do
    [
      ci: ["test --include slow:true --cover", "coveralls.detail"]
    ]
  end
end
